import tensorflow as tf
from srccode.data  import *
from srccode.models import *

def createOptimizer(params):

  lr_schedule = tf.keras.optimizers.schedules.ExponentialDecay(
                   initial_learning_rate = params['LEARNING_RATE'],
                   decay_steps           = params['DECAY_STEPS'],
                   decay_rate            = params['DECAY_RATE'])

  optimizer   = tf.keras.optimizers.Adam(learning_rate=lr_schedule)

  return optimizer

def custom_loss_function(y_true, y_pred):
  return tf.keras.losses.mean_squared_error(y_true*255, y_pred*255)

def custom_metrics_ssim(y_true, y_pred):
  return tf.reduce_mean(tf.image.ssim(y_true, y_pred, max_val=1))

def createModel(params):

  model     = HdrNet(params)
  optimizer = createOptimizer(params)
  model.compile(optimizer = optimizer, 
                loss      = custom_loss_function,
                metrics   = [custom_metrics_ssim])
  model.build(input_shape=(None, None, None, 3))
  model.build_graph().summary()

  return model